@@ -32,63 +32,74 @@
 #include "command.h"
 #include "strlist.h"
 #include "ui.h"
+
+/* compatibility mode flag */
+extern int compat_mode;
+/* Non NULL when -s option specifyed on command line */
+extern const char *fdisk_partsize_device;
+extern int fdisk_partsize_part; /* ?? */
+
+/* not 0 when fdisk should print the partition table? */
+extern int fdisk_list_table;
+
+#if HAVE_LOCALE_H
+# include <locale.h>
+#endif /* HAVE_LOCALE_H */
+
+#include "gettext.h"
 #define N_(String) String
-#if ENABLE_NLS
-#  include <libintl.h>
-#  include <locale.h>
-#  define _(String) dgettext (PACKAGE, String)
-#else
-#  define _(String) (String)
-#endif /* ENABLE_NLS */
+#define _(String) dgettext (PACKAGE, String)
 
-#ifdef HAVE_LIBREADLINE
+/* BEGIN readline stuff */
+#if HAVE_READLINE
 
-#ifdef HAVE_TERMCAP_H
-#include <termcap.h>
-#else
-extern int tgetnum (char* key);
-#endif
+/* Termcap need this buffer to initialize terminal database, I think
+   the db is stored into this buffer but I'm not sure. */
+enum { TERMCAP_BUFFER_SIZE = 2048 };
+static char termcap_buffer[TERMCAP_BUFFER_SIZE] = {0};
 
-#include <readline/readline.h>
-#include <readline/history.h>
+/* Use this flag to know if termcap is propely initialized
+   and ready or should not be used */
+static enum { TERMCAP_OK, TERMCAP_ERROR } termcap_status;
 
-#ifndef HAVE_RL_COMPLETION_MATCHES
-#define rl_completion_matches completion_matches
-#endif
+/* termcap library */
+# if HAVE_TERMCAP_H
+#  include <termcap.h>
+# else
+    extern int tgetnum (char* key);
+# endif /* HAVE_TERMCAP_H */
 
-#ifndef rl_compentry_func_t
-#define rl_compentry_func_t void
-#endif
+/* readline/readline.h */
+# if HAVE_READLINE_READLINE_H
+#  include <readline/readline.h>
+# else
+   extern char * readline ();
+   extern void add_history ();
+# endif /* HAVE_READLINE_READLINE_H */
 
-#endif /* HAVE_LIBREADLINE */
+/* history library */
+# if HAVE_READLINE_HISTORY_H
+#  include <readline/history.h>
+# else
+#  define _add_history_unique add_history
+# endif
+  
+
+# ifndef HAVE_RL_COMPLETION_MATCHES
+#  define rl_completion_matches completion_matches
+# endif
+
+# ifndef rl_compentry_func_t
+#  define rl_compentry_func_t void
+# endif
+
+#endif /* HAVE_READLINE */
+
+/* END readline stuff */
+
+/* global and used in fdisk.c */
 char* interface_name = "GNU Fdisk " VERSION;
 
-static char* banner_msg = N_(
-"Copyright (C) 1998 - 2006 Free Software Foundation, Inc.\n"
-"This program is free software, covered by the GNU General Public License.\n"
-"\n"
-"This program is distributed in the hope that it will be useful, "
-"but WITHOUT ANY WARRANTY; without even the implied warranty of "
-"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
-"GNU General Public License for more details.\n\n");
-
-static char* usage_msg = N_(
-"Usage: fdisk [OPTION]... [DEVICE]\n");
- 
-static char* bug_msg = N_(
-"You found a bug in GNU Fdisk.\n"
-"This may have been fixed in the last version of GNU Parted that you can find at:\n"
-"\thttp://ftp.gnu.org/gnu/parted/\n\n"
-"Please check this version prior to bug reporting.\n\n"
-"If this has not been fixed yet or if you don't know how to check, please email:\n"
-"\tbug-parted@gnu.org\n"
-"or (preferably) file a bug report at:\n"
-"\thttp://parted.alioth.debian.org/bugs/\n\n"
-"Your report should contain the version of this release (%s) along with the\n"
-"error message below, the output of\n"
-"\tparted DEVICE unit co print unit s print\n"
-"and additional information about your setup you consider important.\n");
-
 #define MAX_WORDS	1024
 
 static StrList*		command_line;
@@ -100,172 +111,268 @@ static StrList*		on_off_list;
 static StrList*		fs_type_list;
 static StrList*		disk_type_list;
 
+/* Declare readline_state */
 static struct {
-	const StrList*	possibilities;
-	const StrList*	cur_pos;
-	int		in_readline;
-	sigjmp_buf	jmp_state;
+  const StrList* possibilities;
+  const StrList* cur_pos;
+  int		 in_readline;
+  sigjmp_buf	 jmp_state;
 } readline_state;
 
-static PedExceptionOption	current_exception_opt = 0;
+static PedExceptionOption current_exception_opt = 0;
 
-/* If we don't want the possibilities to appear on the prompt, this 
+/* If we don't want the possibilities to appear on the prompt, this
    should be set to 0. TODO: Making this without a global variable
    might be a good idea */
 static prompt_possibilities = 1;
 
-#ifdef HAVE_LIBREADLINE
+
+#ifdef HAVE_READLINE
+/* This function initialize the termcap library */
+void init_termcap_library(void)
+{
+  char *terminal;
+  int status;
+
+  terminal = getenv("TERM");
+  if(terminal == NULL)
+    {
+      fputs(_("can't get `TERM' environment variable.\n"
+	      "Please specify a terminal type int the `TERM' environment\n"
+	      "variable.\n"), stderr);
+      goto _Error;
+    }
+  status = tgetent(termcap_buffer, terminal);
+  if(status <= 0)
+    {
+      if(status == 0)
+	fprintf(stderr, 
+		_("%s termcap error: terminal `%s' is not defined in your\n"
+		  "terminal database, please update it.\n"),
+		PACKAGE, terminal);
+      else
+	fprintf(stderr, 
+		_("%s termcap error: could not access terminal database\n"
+		  "please check yor termcap library.\n"),
+		PACKAGE);
+      goto _Error;
+    }
+  else
+    termcap_status = TERMCAP_OK;
+  return;
+ 
+ _Error:
+  termcap_status = TERMCAP_ERROR;
+}
+
 /* returns matching commands for text */
 static char*
 command_generator (char* text, int state)
 {
-	if (!state)
-		readline_state.cur_pos = readline_state.possibilities;
+  if (!state)
+    readline_state.cur_pos = readline_state.possibilities;
 
-	while (readline_state.cur_pos) {
-		const StrList*	cur = readline_state.cur_pos;
-		readline_state.cur_pos = cur->next;
-		if (str_list_match_node (cur, text))
-			return str_list_convert_node (cur);
-	}
+  while (readline_state.cur_pos) {
+    const StrList* cur = readline_state.cur_pos;
+    readline_state.cur_pos = cur->next;
+    if (str_list_match_node (cur, text))
+      return str_list_convert_node (cur);
+  }
 
-	return NULL;
+  return NULL;
 }
 
 /* completion function for readline() */
 char**
 fdisk_complete_function (char* text, int start, int end)
 {
-	return rl_completion_matches (text,
-		(rl_compentry_func_t*) command_generator);
+  return rl_completion_matches (text, (rl_compentry_func_t*) command_generator);
 }
 
+#if HAVE_READLINE_HISTORY_H
 static void
 _add_history_unique (const char* line)
 {
-	HIST_ENTRY*	last_entry = current_history ();
-	if (!strlen (line))
-		return;
-	if (!last_entry || strcmp (last_entry->line, line))
-		add_history ((char*) line);
+  HIST_ENTRY** p;
+
+  /* Check if line is empty */
+  if (!line || !line[0])
+    return;
+
+  /* Get all entryes of history and find if current line 
+   * is already in one history entry. */
+  p = history_list();
+  
+  if(p != NULL)
+    /* if the history is not empty we search for line in all entryes */
+    {
+      unsigned int i;
+      for(i = 0; p[i] != NULL; i++)
+	if(strcmp(p[i]->line, line) == 0)
+	  /* match */
+	  return;
+    }
+  /* no match */
+  add_history(line);
 }
-#endif /* HAVE_LIBREADLINE */
+#endif /* HAVE_READLINE_HISTORY_H */
+
+#endif /* HAVE_READLINE */
 
 static void
 interrupt_handler (int signum)
 {
-	int	in_readline = readline_state.in_readline;
+  int in_readline = readline_state.in_readline;
 
-	readline_state.in_readline = 0;
+  readline_state.in_readline = 0;
 
-	signal (signum, &interrupt_handler);
+  signal (signum, &interrupt_handler);
 
-	if (in_readline) {
-		printf ("\n");
-		siglongjmp (readline_state.jmp_state, 1);
-	}
+  if (in_readline) {
+    putchar(0x0A);
+    siglongjmp (readline_state.jmp_state, 1);
+  }
+}
+
+/* check if line is only spaces */
+static int
+is_empty_line(const char *s)
+{
+  while(*s)
+    if(!isspace(*s++))
+      return 0;
+  return 1;
 }
 
 static char*
 _readline (const char* prompt, const StrList* possibilities)
 {
-	char*		line;
+  char* line = NULL;
 
-	readline_state.possibilities = possibilities;
-	readline_state.cur_pos = NULL;
-	readline_state.in_readline = 1;
+  readline_state.possibilities = possibilities;
+  readline_state.cur_pos = NULL;
+  readline_state.in_readline = 1;
 
-	if (sigsetjmp (readline_state.jmp_state, 1))
-		return NULL;
+  if (sigsetjmp (readline_state.jmp_state, 1))
+    return NULL;
 
-	fdisk_wipe_line ();
-#ifdef HAVE_LIBREADLINE
-	if (!fdisk_opt_script_mode) {
-		/* XXX: why isn't prompt const? */
-		line = readline ((char*) prompt);
-		if (line)
-			_add_history_unique (line);
-	} else
-#endif
+  fdisk_wipe_line ();
+
+  if (!fdisk_opt_script_mode)
+#ifdef HAVE_READLINE
+    { /* !fdisk_opt_script_mode && HAVE_READLINE */
+      line = readline(prompt);
+      if(line && *line)
+	_add_history_unique (line);
+    }
+#else
+    { /* !fdisk_opt_script_mode */
+      printf ("%s", prompt);
+      fflush (stdout);
+      line = (char*) calloc(sizeof(char), 256);
+      if (fgets (line, 256, stdin) == NULL || is_empty_line(line))
 	{
-		printf ("%s", prompt);
-		fflush (stdout);
-		line = (char*) malloc (256);
-		if (fgets (line, 256, stdin) && strcmp (line, "") != 0) {
-			line [strlen (line) - 1] = 0;	/* kill trailing CR */
-		} else {
-			free (line);
-			line = NULL;
-		}
+	  free (line);
+	  line = NULL;
 	}
-
-	readline_state.in_readline = 0;
-	return line;
+    }
+#endif
+  /* MMM... I assume that `disk_opt_script_mode' is true when fdisk
+   * would be used whit a script, but this script where come from,
+   * a file or stdin? the following codo works on stdin
+   * but we kan do better with an option and a file... ? */
+  else
+    { /* disk_opt_script_mode */
+      line = (char*) calloc(sizeof(char), 256);
+      if (fgets (line, 256, stdin) == NULL || is_empty_line(line))
+	{
+	  free (line);
+	  line = NULL;
+	}
+    }
+  readline_state.in_readline = 0;
+  return line;
 }
 
 static PedExceptionOption
 option_get_next (PedExceptionOption options, PedExceptionOption current)
 {
-	PedExceptionOption	i;
+  PedExceptionOption i;
 
-	if (current == 0)
-		i = PED_EXCEPTION_OPTION_FIRST;
-	else
-		i = current * 2;
+  i = 
+    current == 0 ? PED_EXCEPTION_OPTION_FIRST : current * 2;
 
-	for (; i <= options; i *= 2) {
-		if (options & i)
-			return i;
-	}
+#if 0
+  if (current == 0)
+    i = PED_EXCEPTION_OPTION_FIRST;
+  else
+    i = current * 2;
+#endif 
 
-	return 0;
+  for (; i <= options; i *= 2) 
+    {
+      if (options & i)
+	return i;
+    }
+  return 0;
 }
 
 static void
 _print_exception_text (PedException* ex)
 {
-	StrList*		text;
+  StrList*		text;
+  char* bug_msg = _(
+"You found a bug in GNU Fdisk.\n\
+This may have been fixed in the last version of \
+GNU Parted that you can find at:\n\
+\thttp://ftp.gnu.org/gnu/parted/\n\n\
+Please check this version prior to bug reporting.\n\n\
+If this has not been fixed yet or if you don't \
+know how to check, please email:\n\
+\tbug-parted@gnu.org\n\
+or (preferably) file a bug report at:\n\
+\thttp://parted.alioth.debian.org/bugs/\n\n\
+Your report should contain the version of this release (%s) along with the\n\
+error message below, the output of\n\
+\tparted DEVICE unit co print unit s print\n\
+and additional information about your setup you consider important.\n");
 
-	fdisk_wipe_line ();
+  fdisk_wipe_line ();
 
-	if (ex->type == PED_EXCEPTION_BUG) {
-		printf (bug_msg, VERSION);
-		text = str_list_create ("\n", ex->message, "\n", NULL);
-	} else {
-		text = str_list_create (
-			   _(ped_exception_get_type_string (ex->type)),
-			   ": ", ex->message, "\n", NULL);
-	}
-
-	str_list_print_wrap (text, fdisk_screen_width (), 0, 0);
-	str_list_destroy (text);
+  if (ex->type == PED_EXCEPTION_BUG) 
+    {
+      printf (bug_msg, VERSION);
+      text = str_list_create ("\n", ex->message, "\n", NULL);
+    } 
+  else 
+    text = str_list_create (_(ped_exception_get_type_string (ex->type)),
+			    ": ", ex->message, "\n", NULL);
+  str_list_print_wrap (text, fdisk_screen_width (), 0, 0);
+  str_list_destroy (text);
 }
 
 static PedExceptionOption
 exception_handler (PedException* ex)
 {
-	PedExceptionOption	opt;
+  PedExceptionOption opt;
 
-	_print_exception_text (ex);
+  _print_exception_text (ex);
 
-	/* only one choice?  Take it ;-) */
-	opt = option_get_next (ex->options, 0);
-	if (!option_get_next (ex->options, opt))
-		return opt;
+  /* only one choice?  Take it ;-) */
+  opt = option_get_next (ex->options, 0);
+  if (!option_get_next (ex->options, opt))
+    return opt;
 
-	/* script-mode: don't handle the exception */
-	if (fdisk_opt_script_mode)
-		return PED_EXCEPTION_UNHANDLED;
+  /* script-mode: don't handle the exception */
+  if (fdisk_opt_script_mode)
+    return PED_EXCEPTION_UNHANDLED;
 
-	do {
-		opt = fdisk_command_line_get_ex_opt ("", ex->options);
-	} while (opt == PED_EXCEPTION_UNHANDLED && isatty (0));
-	return opt;
+  do {
+    opt = fdisk_command_line_get_ex_opt ("", ex->options);
+  } while (opt == PED_EXCEPTION_UNHANDLED && isatty (0));
+  return opt;
 }
 
-
-
-
 static int
 _str_is_spaces (const char* str)
 {
@@ -277,250 +384,385 @@ _str_is_spaces (const char* str)
 static char*
 realloc_and_cat (char* str, const char* append)
 {
-	int	length = strlen (str) + strlen (append) + 1;
-	char*	new_str = realloc (str, length);
-	strcat (new_str, append);
-	return new_str;
+  int length = strlen (str) + strlen (append) + 1;
+  char*	new_str = realloc (str, length);
+  strcat (new_str, append);
+  return new_str;
 }
 
 static char*
 _construct_prompt (const char* head, const char* def,
 		   const StrList* possibilities)
 {
-	char*	prompt = strdup (head);
+  char*	prompt = strdup (head);
 
-	if (def && possibilities)
-		PED_ASSERT (str_list_match_any (possibilities, def),
-			    return NULL);
+  if (def && possibilities)
+    PED_ASSERT (str_list_match_any (possibilities, def), return NULL);
 
-	if (possibilities && prompt_possibilities && 
-	    str_list_length (possibilities) < 8) {
-		const StrList*	walk;
-		if (strlen (prompt))
-			prompt = realloc_and_cat (prompt, " ");
-
-		for (walk = possibilities; walk; walk = walk->next) {
-			if (walk != possibilities)
-				prompt = realloc_and_cat (prompt, "/");
-
-			if (def && str_list_match_node (walk, def) == 2) {
-				prompt = realloc_and_cat (prompt, "[");
-				prompt = realloc_and_cat (prompt, def);
-				prompt = realloc_and_cat (prompt, "]");
-			} else {
-				char*	text = str_list_convert_node (walk);
-				prompt = realloc_and_cat (prompt, text);
-				free (text);
-			}
-		}
-		prompt = realloc_and_cat (prompt, ": ");
-	} else if (def) {
-		size_t len = strlen (prompt);
-		if (len && prompt[len-1] != '\n')
-			prompt = realloc_and_cat (prompt, "  ");
-		prompt = realloc_and_cat (prompt, "(default ");
-		prompt = realloc_and_cat (prompt, def);
-		prompt = realloc_and_cat (prompt, "): ");
-	} else {
-		size_t len = strlen (prompt);
-		if (len && prompt[--len] != '\n') {
-			if (prompt[len] == ':' || prompt[len] == '?')
-				prompt = realloc_and_cat (prompt, " ");
-			else
-				prompt = realloc_and_cat (prompt, ": ");
-		}
+  if (possibilities &&
+      prompt_possibilities &&
+      str_list_length (possibilities) < 8) 
+    {
+      const StrList* walk;
+      if (strlen (prompt))
+	prompt = realloc_and_cat (prompt, " ");
+      
+      for (walk = possibilities; walk; walk = walk->next) 
+	{
+	  if (walk != possibilities)
+	    prompt = realloc_and_cat (prompt, "/");
+	  if (def && str_list_match_node (walk, def) == 2) 
+	    {
+	      prompt = realloc_and_cat (prompt, "[");
+	      prompt = realloc_and_cat (prompt, def);
+	      prompt = realloc_and_cat (prompt, "]");
+	    } 
+	  else 
+	    {
+	      char* text = str_list_convert_node (walk);
+	      prompt = realloc_and_cat (prompt, text);
+	      free (text);
+	    }
 	}
-
-	return prompt;
+      prompt = realloc_and_cat (prompt, ": ");
+    } 
+  else if (def) 
+    {
+      size_t len = strlen (prompt);
+      if (len && prompt[len-1] != '\n')
+	prompt = realloc_and_cat (prompt, "  ");
+      prompt = realloc_and_cat (prompt, "(default ");
+      prompt = realloc_and_cat (prompt, def);
+      prompt = realloc_and_cat (prompt, "): ");
+    } 
+  else 
+    {
+      size_t len = strlen (prompt);
+      if (len && prompt[--len] != '\n') 
+	{
+	  if (prompt[len] == ':' || prompt[len] == '?')
+	    prompt = realloc_and_cat (prompt, " ");
+	  else
+	    prompt = realloc_and_cat (prompt, ": ");
+	}
+    }
+  return prompt;
 }
 
 static int
 _can_create_primary (const PedDisk* disk)
 {
-	int	i;
+  int i;
 
-	for (i = 1; i <= ped_disk_get_max_primary_partition_count (disk); i++) {
-		if (!ped_disk_get_partition (disk, i))
-			return 1;
-	}
-
-	return 0;
+  for (i = 1; i <= ped_disk_get_max_primary_partition_count (disk); i++) 
+    {
+      if (!ped_disk_get_partition (disk, i))
+	return 1;
+    }
+  return 0;
 }
 
 static int
 _can_create_extended (const PedDisk* disk)
 {
-	if (!_can_create_primary (disk))
-		return 0;
-	if (!ped_disk_type_check_feature (disk->type, PED_DISK_TYPE_EXTENDED))
-		return 0;
-	if (ped_disk_extended_partition (disk))
-		return 0;
-	return 1;
+  if (!_can_create_primary (disk))
+    return 0;
+  if (!ped_disk_type_check_feature (disk->type, PED_DISK_TYPE_EXTENDED))
+    return 0;
+  if (ped_disk_extended_partition (disk))
+    return 0;
+  return 1;
 }
 
 static int
 _can_create_logical (const PedDisk* disk)
 {
-	if (!ped_disk_type_check_feature (disk->type, PED_DISK_TYPE_EXTENDED))
-		return 0;
-	return ped_disk_extended_partition (disk) != 0;
+  if (!ped_disk_type_check_feature (disk->type, PED_DISK_TYPE_EXTENDED))
+    return 0;
+  return ped_disk_extended_partition (disk) != 0;
 }
 
 static int
 init_ex_opt_str ()
 {
-	int			i;
-	PedExceptionOption	opt;
+  int i;
+  PedExceptionOption opt;
 
-	for (i = 0; (1 << i) <= PED_EXCEPTION_OPTION_LAST; i++) {
-		opt = (1 << i);
-		ex_opt_str [i]
-			= str_list_create (
-				ped_exception_get_option_string (opt),
-				_(ped_exception_get_option_string (opt)),
-				NULL);
-		if (!ex_opt_str [i])
-			return 0;
-	}
-
-	ex_opt_str [i] = NULL;
-	return 1;
+  for (i = 0; (1 << i) <= PED_EXCEPTION_OPTION_LAST; i++) 
+    {
+      opt = (1 << i);
+      ex_opt_str[i] = 
+	str_list_create (ped_exception_get_option_string (opt),
+			 _(ped_exception_get_option_string (opt)), NULL);
+      if (!ex_opt_str [i])
+	return 0;
+    }
+  ex_opt_str [i] = NULL;
+  return 1;
 }
 
 static void
 done_ex_opt_str ()
 {
-	int	i;
+  int i;
 
-	for (i=0; ex_opt_str [i]; i++)
-		str_list_destroy (ex_opt_str [i]);
+  for (i=0; ex_opt_str [i]; i++)
+    str_list_destroy (ex_opt_str [i]);
 }
 
 static int
 init_state_str ()
 {
-	on_list = str_list_create_unique (_("on"), "on", NULL);
-	off_list = str_list_create_unique (_("off"), "off", NULL);
-	on_off_list = str_list_join (str_list_duplicate (on_list),
-				     str_list_duplicate (off_list));
-	return 1;
+  on_list = str_list_create_unique (_("on"), "on", NULL);
+  off_list = str_list_create_unique (_("off"), "off", NULL);
+  on_off_list = str_list_join (str_list_duplicate (on_list),
+			       str_list_duplicate (off_list));
+  return 1;
 }
 
 static void
 done_state_str ()
 {
-	str_list_destroy (on_list);
-	str_list_destroy (off_list);
-	str_list_destroy (on_off_list);
+  str_list_destroy (on_list);
+  str_list_destroy (off_list);
+  str_list_destroy (on_off_list);
 }
 
 static int
 init_fs_type_str ()
 {
-	PedFileSystemType*	walk;
+  PedFileSystemType* walk;
 
-	fs_type_list = NULL;
+  fs_type_list = NULL;
 
-	for (walk = ped_file_system_type_get_next (NULL); walk;
-	     walk = ped_file_system_type_get_next (walk))
-	{
-		fs_type_list = str_list_insert (fs_type_list, walk->name);
-		if (!fs_type_list)
-			return 0;
-	}
-
-	return 1;
+  for (walk = ped_file_system_type_get_next (NULL); walk;
+       walk = ped_file_system_type_get_next (walk))
+    {
+      fs_type_list = str_list_insert (fs_type_list, walk->name);
+      if (!fs_type_list)
+	return 0;
+    }
+  return 1;
 }
 
 static int
 init_disk_type_str ()
 {
-	PedDiskType*	walk;
+  PedDiskType* walk;
 
-	disk_type_list = NULL;
+  disk_type_list = NULL;
 
-	for (walk = ped_disk_type_get_next (NULL); walk;
-	     walk = ped_disk_type_get_next (walk))
-	{
-		disk_type_list = str_list_insert (disk_type_list, walk->name);
-		if (!disk_type_list)
-			return 0;
-	}
-
-	return 1;
+  for (walk = ped_disk_type_get_next (NULL); walk;
+       walk = ped_disk_type_get_next (walk))
+    {
+      disk_type_list = str_list_insert (disk_type_list, walk->name);
+      if (!disk_type_list)
+	return 0;
+    }
+  return 1;
 }
 
-void 
-fdisk_usage_msg() {
-        printf (_(usage_msg));
+void
+fdisk_usage_msg()
+{
+  char *s = _("Usage: fdisk [OPTION]... [DEVICE]");
+  puts(s);
 }
 
 void
 fdisk_help_msg ()
 {
-        fdisk_usage_msg();
+  fdisk_usage_msg();
 
-	printf ("\n%s\n", _("OPTIONs:"));
-	fdisk_print_options_help ();
+  printf ("\n%s\n", _("OPTIONs:"));
+  fdisk_print_options_help ();
 /*
 	printf ("\n%s\n", _("COMMANDs:"));
 	fdisk_print_commands_help ((FdiskCommand**)NULL);
 */
-	exit (0);
+  exit (0);
 }
 
-int
-fdisk_interactive_mode (PedDisk** disk, FdiskCommand* cmd_list[])
+/* move all in a function so we reduce stack size */
+static void
+do_banner_message(void)
 {
-	char*		line;
-	StrList*	list;
-	StrList*	command_names = fdisk_command_get_names (cmd_list);
+  char* banner_msg = 
+"Copyright (C) 1998 - 2006 Free Software Foundation, Inc.\n\
+This program is free software, covered by the GNU General Public License.\n\
+\n\
+This program is distributed in the hope that it will be useful,\n\
+but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
+GNU General Public License for more details.\n\n";
 
-	printf ("%s\n", interface_name);
+  StrList* list;
+  
+  list = str_list_create (_(banner_msg), NULL);
+  str_list_print_wrap (list, fdisk_screen_width (), 0, 0);
+  str_list_destroy (list);
+}
 
-	list = str_list_create (_(banner_msg), NULL);
-	str_list_print_wrap (list, fdisk_screen_width (), 0, 0);
-	str_list_destroy (list);
+/*
+ * Begin interactive mode
+ */
 
-	fdisk_print_using_dev ((*disk)->dev);
+/* Changed */
+/* 
+int
+fdisk_interactive_mode (PedDisk** disk, FdiskCommand* cmd_list[]) 
+*/
 
-	while (1) {
-		char*		word;
-		FdiskCommand*	cmd;
+int
+fdisk_interactive_mode(PedDevice **dev, FdiskCommand* cmd_list[])
+{
+  char*	line;
+  PedDisk *disk;
+  StrList* list;
+  StrList* command_names = fdisk_command_get_names (cmd_list);
+    
+  puts (interface_name);
+  do_banner_message();
 
-		commands = cmd_list;	/* FIXME yucky, nasty, evil hack */
-
-		while (!fdisk_command_line_get_word_count ()) {
-			if (feof (stdin)) {
-				printf ("\n");
-				return 1;
-			}
-			prompt_possibilities = 0;
-			fdisk_command_line_prompt_words ("Command (m for help):", NULL,
-							 command_names, 1);
-			prompt_possibilities = 1;
-		}
-
-		word = fdisk_command_line_pop_word ();
-		if (word) {
-			cmd = fdisk_command_get (commands, word);
-			free (word);
-			if (cmd) {
-				if (!fdisk_command_run (cmd, disk))
-					fdisk_command_line_flush ();
-			} else {
-				fdisk_print_commands_help (commands);
-			}
-		}
+  /* Christian <mail.kristian@yahoo.it>:
+   * FIRST try open device as a disk with a partition table, if this doesn't
+   * work device has not partition table so we must make it */
+ 
+   
+  if(ped_disk_probe(*dev) == NULL)
+    {
+      if(compat_mode)
+	{
+	  /* If we are in compat mode we must create a dos partition table */
+	  puts( _("Device contains neither a valid DOS partition table, "
+		  "nor Sun, SGI or OSF disklabel\n"
+		  "Building a new DOS disklabel. Changes will remain in memory only,\n"
+		  "until you decide to write them. After that, of course, the previous\n"
+		  "content won't be recoverable.\n"));
+	  if((disk = ped_disk_new_fresh(*dev, ped_disk_type_get("msdos"))) == NULL)
+	    return EXIT_FAILURE;
 	}
+      else
+	{
+	  /* In extended mode we can create a partition table with all supported modes */
+	  PedDiskType *label_type;
+	  StrList *supp = NULL;
+	  char *supported_types;
+	  char *str;
 
-	return 1;
+	  for(label_type = ped_disk_type_get_next(NULL);
+	      label_type = ped_disk_type_get_next(label_type); )
+	    {
+	      supp = str_list_append(supp, label_type->name);
+	      supp = str_list_append(supp, " ");
+	    }
+
+	  supported_types = str_list_convert(supp);
+	  str_list_destroy(supp);
+
+	  /* List resources was freed */
+
+	  puts(_("Device contains neither a valid DOS partition table, "
+		 "nor Sun, SGI or OSF disklabel.\n"
+		 "You must create a disk label first!\n"));
+	  
+	  str = malloc(32);
+
+	get_label_type_str:
+	  
+	  printf(_("What type of disk label would you create (q to quit)?\n[ %s] "), 
+		 supported_types);
+
+	retry_without_print_message:
+	  
+	  if(scanf("%31s", str) < 1)
+	    {
+	      goto retry_without_print_message;
+	    }
+	  else if((*str == 'q' || *str == 'Q') && str[1] == 0)
+	    {
+	      return EXIT_SUCCESS;
+	    }
+	  else if((label_type = ped_disk_type_get(str)) == NULL)
+	    {
+	      fputs(_("*** Invalid label!\n"), stderr);
+	      goto get_label_type_str;	      
+	    }
+
+	  disk = ped_disk_new_fresh(*dev, label_type);
+
+	  if(!disk)
+	    return EXIT_FAILURE; /* what bad? :( */
+
+	  /* Free other resources */
+	  free(str);
+	  free(supported_types);
+	  
+	}
+    } /* if(!disk) */
+  else
+    {
+      disk = ped_disk_new(*dev);
+      if(!disk)
+	return EXIT_FAILURE;
+    }
+  
+  /***
+   ** OK, We have a disk with a partition table :) 
+   */
+  
+  /* Show the size of the partition */
+  if(fdisk_partsize_device)
+    fdisk_print_partition_size(disk);
+  
+  /* List the specified disk. */
+  if (fdisk_list_table)
+    fdisk_do_list_devices(&disk);
+  
+
+  fdisk_print_using_dev (disk->dev);
+
+  while (1) 
+    {
+      char* word;
+      FdiskCommand* cmd;
+
+      commands = cmd_list;	/* FIXME yucky, nasty, evil hack */
+
+      while (!fdisk_command_line_get_word_count ()) 
+	{
+	  if (feof (stdin))
+	    {
+	      printf ("\n");
+	      return 1;
+	    }
+	  prompt_possibilities = 0;
+	  fdisk_command_line_prompt_words ("Command (m for help):", 
+					   NULL, command_names, 1);
+	  prompt_possibilities = 1;
+	}
+      word = fdisk_command_line_pop_word ();
+      if (word) 
+	{
+	  cmd = fdisk_command_get (commands, word);
+	  free (word);
+	  if (cmd) 
+	    {
+	      if (!fdisk_command_run (cmd, &disk))
+		fdisk_command_line_flush ();
+	    } 
+	  else 
+	    {
+	      fdisk_print_commands_help (commands);
+	    }
+	}
+    }
+  return 1;
 }
 
 /* If menu equals:
  * 0 - expert menu.
- * 1 - filesystem menu. 
+ * 1 - filesystem menu.
  * 2 - bsd menu
  */
 int
@@ -533,7 +775,7 @@ fdisk_interactive_menu (PedDisk** disk, FdiskCommand* cmd_list[], int menu)
 	/* FIXME yucky, nasty, evil hack */
 	FdiskCommand **saved_commands = commands;
 	commands = cmd_list;
-	
+
 
 	while (1) {
 		char*		word;
@@ -597,20 +839,18 @@ command_line_prompt_boolean_question (const char* prompt) {
 int
 fdisk_screen_width ()
 {
-	int	width = 0;
-
-	if (fdisk_opt_script_mode)
-		return 32768;	/* no wrapping ;) */
+  int width = 0;
+  if (fdisk_opt_script_mode)
+    return 32768;	/* no wrapping ;) */
 
 /* HACK: don't specify termcap separately - it'll annoy the users. */
-#ifdef HAVE_LIBREADLINE
-	width = tgetnum ("co");
+#ifdef HAVE_READLINE
+  if(termcap_status == TERMCAP_OK)
+    width = tgetnum ("co");
 #endif
-
-	if (width <= 0)
-		width = 80;
-
-	return width;
+  if (width <= 0)
+    width = 80;
+  return width;
 }
 
 void
@@ -634,7 +874,7 @@ fdisk_command_line_get_ex_opt (const char* prompt, PedExceptionOption options)
 	const char*		opt_name;
 	int			i = 0, result;
 
-	for (opt = option_get_next (options, 0); opt && i < sizeof(optlist) - 1; 
+	for (opt = option_get_next (options, 0); opt && i < sizeof(optlist) - 1;
 	     opt = option_get_next (options, opt), i++) {
 		opt_name = ped_exception_get_option_string (opt);
 		optlist[i].option = tolower(opt_name[0]);
@@ -646,10 +886,10 @@ fdisk_command_line_get_ex_opt (const char* prompt, PedExceptionOption options)
 	if (!result)
 		return PED_EXCEPTION_UNHANDLED;
 
-	for (opt = option_get_next (options, 0); opt; 
+	for (opt = option_get_next (options, 0); opt;
 	     opt = option_get_next (options, opt)) {
 		opt_name = ped_exception_get_option_string (opt);
-		if (result == tolower(opt_name[0])) 
+		if (result == tolower(opt_name[0]))
 			break;
 	}
 	return opt;
@@ -973,7 +1213,7 @@ fdisk_command_line_get_option (const char* head, const Option* opts)
 	StrList *possibilities = NULL;
 	char *prompt = strdup(head);
 	char *result;
-	char buf[128]; 
+	char buf[128];
 	int i;
 
 	buf[1] = '\0';
@@ -997,14 +1237,14 @@ fdisk_command_line_get_option (const char* head, const Option* opts)
 	/* We add a newline at the end, so _construct_prompt
            handles it as it should */
 	prompt = realloc_and_cat (prompt, "\n");
-	
+
 
 	/* Queries the user about the option, we turn off the automatic
 	   adding of possibilities to the prompt */
 	prompt_possibilities = 0;
 	result = fdisk_command_line_get_word (prompt, NULL, possibilities, 1);
 	prompt_possibilities = 1;
-	
+
 	/* We can use i, not very intuitive to the coder, but who cares */
 	if (result) {
 		i = result[0];
@@ -1015,12 +1255,12 @@ fdisk_command_line_get_option (const char* head, const Option* opts)
 	}
 	free(prompt);
 	return i;
-} 
+}
 
 int
 fdisk_get_partpos (const char* prompt, const void* context, const char *possibilities)
 {
-	
+
 	const Option *orig_opts = (Option *) context;
 	Option opts[5];
 	int i,j;
@@ -1032,7 +1272,7 @@ fdisk_get_partpos (const char* prompt, const void* context, const char *possibil
 		}
 	}
 	opts[j].option = 0;
-	
+
 	return fdisk_command_line_get_option(prompt, opts);
 }
 int
@@ -1055,7 +1295,7 @@ fdisk_command_line_get_part_type (const char* prompt, const PedDisk *disk,
 	if (_can_create_primary (disk)) {
 		opts[i].option = 'p';
 		opts[i].description = _("primary partition (1-4)");
-		i++; 
+		i++;
 	}
 	opts[i].option = 0;
 	if (i == 0)
@@ -1184,34 +1424,37 @@ fdisk_command_line_get_sector (const char* prompt, PedDevice* dev, PedSector* va
 int
 fdisk_init_ui ()
 {
-	fdisk_opt_script_mode = !isatty (0);
+/* Readline support */
 
-	if (!init_ex_opt_str ()
-	    || !init_state_str ()
-	    || !init_fs_type_str ()
-	    || !init_disk_type_str ())
-		return 0;
-	ped_exception_set_handler (exception_handler);
+/* End readline support */
+  fdisk_opt_script_mode = !isatty (0);
+  
+  if (!init_ex_opt_str ()
+      || !init_state_str ()
+      || !init_fs_type_str ()
+      || !init_disk_type_str ())
+    return 0;
+  ped_exception_set_handler (exception_handler);
 
-#ifdef HAVE_LIBREADLINE
-	rl_initialize ();
-	rl_attempted_completion_function = (CPPFunction*) fdisk_complete_function;
-	readline_state.in_readline = 0;
-#endif
+#ifdef HAVE_READLINE
+  rl_initialize ();
+  rl_attempted_completion_function = (CPPFunction*) fdisk_complete_function;
+  readline_state.in_readline = 0;
+#endif /* HAVE_READLINE */
 
-	signal (SIGINT, &interrupt_handler);
+  signal (SIGINT, &interrupt_handler);
 
-	return 1;
+  return 1;
 }
 
 void
 fdisk_done_ui ()
 {
-	ped_exception_set_handler (NULL);
-	done_ex_opt_str ();
-	done_state_str ();
-	str_list_destroy (fs_type_list);
-	str_list_destroy (disk_type_list);
+  ped_exception_set_handler (NULL);
+  done_ex_opt_str ();
+  done_state_str ();
+  str_list_destroy (fs_type_list);
+  str_list_destroy (disk_type_list);
 }
 
 int
